#define __NQ_INTERNAL_SW__

#include "PlatformSW.h"
#include "NQDeclarations.h"
#include "NQLogging.h"
#include "NQLockMechanism.h"
#include <assert.h>

/*
	CLockMechanism - SW (Nov 2006 to Mar 2007)

	Implementation of a platform independent mutex object in order to synchronize multiple thread 
	or multiple task access.

	Used inside ResponseWaitQueue, PacketQueue, MessageBuffers
 */

CNQLockMechanism::CNQLockMechanism() : m_hMutex(NULL)
{ 
	m_hMutex = platformCreateMutex();
	m_uiTimeoutMsec = 1000; // now: 1 second  
							// on windows, this is infinitely repeated until success or clear error 
							// (!= timeout) message; see platformWaitForMutex (windows version)
							
}
     
CNQLockMechanism::~CNQLockMechanism() 
{ 
	if (m_hMutex != NULL) platformDeleteMutex( m_hMutex ); 
}


void CNQLockMechanism::setTimeoutMsec( UINT32 timeout )
{
	m_uiTimeoutMsec = timeout;
}

UINT32 CNQLockMechanism::getTimeoutMsec()
{
	return m_uiTimeoutMsec;
}

//! \retval true    The object has been locked.
//! \retval false   The object could not be locked (f.ex. got timeout).   
bool CNQLockMechanism::lock ()
{
	if ( m_hMutex == NULL ) return false;

	bool ret = platformWaitForMutex(m_hMutex, m_uiTimeoutMsec);

	NQ_DEBUGMODE_ONLY(if (ret == false) NQ_DBG(("ERROR! LockMechanism::lock returned %d\n", ret)));
	assert( ret == true );

	return ret;
}
     
void CNQLockMechanism::unlock () 
{ 
	if (m_hMutex != NULL) 
	{
		bool ret = true;

		ret = platformSignalMutex(m_hMutex);

		NQ_DEBUGMODE_ONLY(if (ret == false) NQ_DBG(("ERROR! LockMechanism::unlock (signal mutex) returned %d\n", ret)));
		
		assert( ret == true );

	}
}

