#ifndef __NQLOCKMECHANISM_H_INCLUDED__
#define __NQLOCKMECHANISM_H_INCLUDED__

/*! \brief Synchronization Base Object  
	
    Implementation of a platform independent mutex object in order to synchronize multiple thread 
	or multiple task access.   
	Used inside ResponseWaitQueue, PacketQueue, MessageBuffers
    
      The receiver will acknowlegde the command by calling Ack().
*/
class CNQLockMechanism
{
	public:
        //! Constructor
					CNQLockMechanism();
        //! Destructor                          
		virtual     ~CNQLockMechanism();
                
        //! Lock the Object
		bool		lock();             
        //! Unlock the Object
		void		unlock();
        
        //! set the timeout constant 
		void		setTimeoutMsec( UINT32 timeout );
        //! read back the timeout constant 
		UINT32	    getTimeoutMsec();

	protected:
        //! platform handle; used to lock/unlock object
		HANDLE	m_hMutex;                    
		UINT32	m_uiTimeoutMsec;
};

#endif // __NQLOCKMECHANISM_H_INCLUDED__
