
#include "NQ.h"

#ifndef LWIP
  #error This software requires the Lightweight IP Software Component.
#endif

#ifndef __ucosii__
  #error This software requires the UCOS II IP Software Component.
#endif

using namespace std;

extern void SimulateRemotePeer(const char *aIPAddressOtherPeer, UINT16 aPortOtherPeer, UINT16 aPort);
extern void SimulateLocalPeer(const char *aIPAddressOtherPeer, UINT16 aPortOtherPeer, UINT16 aPort);

void LocalPeerStart()
{
  printf("Simulating (nios2 109 (vs. oreade20 177)) local peer...\n");
  SimulateLocalPeer("141.34.30.177", 39900, 32900);
}

void RemotePeerStart()
{
  printf("Simulating (nios2 109 (va. ...)) remote peer...\n");
  printf("No remote peer implemented.\n");
}

void main_task(void*)
{
	printf("Nios2 Main() starting up...\n");
	
	//RemotePeerStart();
  LocalPeerStart();
  
  OSTaskDel(OS_PRIO_SELF);
}


#ifdef LCD_DISPLAY_NAME  
FILE* lcdDevice;
#endif /* LCD_DISPLAY_NAME */

/*
 * A MicroC/OS-II message box will be used to communicate between telnet
 * and board LED control tasks.
 */
OS_EVENT *attained_ip_address_sem;

/*
 * tcpip_init_done()
 * 
 * This function is called once the LWIP stack is alive, as specified in our
 * lwip_stack_init() call in main(), below.
 *
 * We tell it about each of our network devices and start the application & 
 * timer threads as appropriate.
 */
static void tcpip_init_done(void *arg)
{
  /* 
   * At this point LWIP has been initialized, but the Ethernet interface has
   * not; the initialise_lwip_devices() call does so, adding in MicroC-OS/II
   * threads for low-level Ethernet MAC interface and TCP protocol timer.
   */
  if (!lwip_devices_init(ETHER_PRIO_SW))
    die_with_error("[tcpip_init_done] Fatal: Can't add ethernet interface!");
    
    attained_ip_address_sem = OSSemCreate(1);
  
  /* 
   * If DHCP is enabled, activate a thread for the 120-second long time period
   * that is allowed to attempt IP address aquisistion via DHCP. The 
   * dchp_timeout_task() routine is in this example design's
   * "network_utilities.c" file.
   */
#if LWIP_DHCP == 1
  if (!sys_thread_new(dhcp_timeout_task, NULL, DHCP_TMR_PRIO_SW))
    die_with_error("[tcpip_init_done] Fatal: Can't add DHCP timer task!");
#endif /* LWIP_DHCP */
  
  /* 
   * Add any application task(s) that rely on LWIP:
   *  
   * http_task() -- Creates HTTP listening sockets, manages incoming
   *                connection requests, and handles all web server duties.
   */
  if(!sys_thread_new(main_task, NULL, MAIN_PRIO_SW))
    die_with_error("[tcpip_init_done] Fatal: Can't add MAIN task!");

}

int main ()
{
  printf("\n\nNetworkQueue(NIOS2/ucOS-II/lwip) starting up\n\n");
  //printf("FD_SETSIZE=%d\n",FD_SETSIZE);
  /* 
   * Our very first call in an LWIP example design is to start up the LWIP 
   * stack. We specify a MicroC/OS-II thread priority for the tcp_ip thread,
   * as well as a call-back routine, tcpip_init_done(), which is called once
   * the stack is alive and well.
   */
  lwip_stack_init(TCPIP_PRIO_SW, tcpip_init_done, 0);
  
  /*
   * As with all MicroC/OS-II designs, once the initial thread(s) and 
   * associated RTOS resources are declared, we start the RTOS. That's it!
   */
  OSStart();

  return 0;
}
