#ifndef __USER_H__
#define __USER_H__
/******************************************************************************
* Copyright  2004 Altera Corporation, San Jose, California, USA.             *
* All rights reserved. All use of this software and documentation is          *
* subject to the License Agreement located at the end of this file below.     *
*******************************************************************************
* Author - PRR/JRK                                                            *
*                                                                             *
* File: user.h                                                                *
*                                                                             *
* Headers for our application. HTTP server-specific headers can be found in   *
* file http.c.                                                                *
*                                                                             *
* Please refer to file readme.txt for notes on this software example.         *
******************************************************************************/

#include "lwipopts.h"

/*
 * Prototypes:
 *    die_with_error() - Kills current task and delivers error message to 
 *                       STDERR.
 * 
 * dhcp_timeout_task() - Keeps track of whether an IP address has been 
 *                       aquired from a DHCP server, or times out and resorts
 *                       to a static IP address.
 * 
 *         http_task() - Manages HTTP connections and calls relevant 
 *                       subroutines to service HTTP requests.
 */
void main_task();
void die_with_error(char err_msg[]);
void dhcp_timeout_task();

/*
 * Time we wait for DHCP to assign an IP address before settling on a static
 * address (default is 120 seconds).
 */
#define DHCP_TIMEOUT ((120 * 1000)/100)

/* 
 * The IP, gateway, and subnet mask address below are used as a last resort if 
 * if no network settings can be found, and DHCP (if enabled) fails. You can
 * edit these as a quick-and-dirty way of changing network settings if desired.
 * 
 * Default fall-back address:
 *           IP: 10.0.0.51
 *      Gateway: 10.0.0.255
 *  Subnet Mask: 255.255.255.0
 */
#define IPADDR0   141
#define IPADDR1   34
#define IPADDR2   30
#define IPADDR3   109

#define GWADDR0   141
#define GWADDR1   34
#define GWADDR2   30
#define GWADDR3   1

#define MSKADDR0  255
#define MSKADDR1  255
#define MSKADDR2  255
#define MSKADDR3  0

/*
 * Task priorities
 *
 * MicroC/OS-II only allows one task (thread) per priority number. Our web
 * SERVER task is given a high priority (lower only than timers which must run 
 * when they need to) so that we can focus on pushing data *out* of the system.
 * An ethernet CLIENT application would have lower priotitization than the 
 * stack & ethernet tasks.
 */

/*
 * now in declarations.h
#define DHCP_TMR_PRIO     1
#define SENDTASK_PRIO     4
#define TCPIP_PRIO        5
#define ETHER_PRIO        6 
// SW! recvtask_prio and sendtask_prio see platform.h
#define RECVTASK_PRIO     7
#define MAIN_PRIO         8
*/

/* usual
#define DHCP_TMR_PRIO     1
#define TCPIP_PRIO        5
#define ETHER_PRIO        6 
#define RECVTASK_PRIO     7
#define SENDTASK_PRIO     8
#define MAIN_PRIO         9
*/

/* 
 * Buffer size for a routine to call if things fail
 */
#define DIE_WITH_ERROR_BUFFER 256

#endif /* __USER_H__ */
