#ifndef __NQPACKETQUEUE_H_INCLUDED__
#define __NQPACKETQUEUE_H_INCLUDED__

#include "PlatformSW.h"
#include "NQLockMechanism.h"
#include "NQDeclarations.h"
#include "NQMessage.h"


//	CNQPacketQueue - SW (Nov 2006 to Mar 2007)
//	+ SW (Jul 2007 to Aug 2007)

/**  \brief	Implementation of platform (and stack) independent FIFO packet queue for objects of type CNQMessage
	
	Using C++ STL this could be done much more easily, but implementation on Nios2 platform did not work
	(reason not fully known, stack issues suspected). This CNQPacketQueue was created as a slightly inferior, but 
	working solution.

	Implementation details:
		- double array linked list
		- access protected by binary semaphore or mutex object
		- no data is copied, only pointer reference
		- enhanced error handling
*/
class CNQPacketQueue : public CNQLockMechanism
{ 
	friend class CNQMessage; // (-x-)
	public:
		CNQPacketQueue();
		virtual ~CNQPacketQueue();

		bool setQueueMaxsize( INT32 maxsize, UINT32 &lasterr );
		UINT32 getLastError();

		// full error code back functions
		bool pop_front( CNQMessage **obj, UINT32 &lasterr);
		bool push_back ( const CNQMessage *obj, UINT32 &lasterr );
		bool remove_elem( UINT16 requestid, UINT32 &lasterr);
		bool find_elem( UINT16 requestid, CNQMessage **obj, UINT32 &lasterr );
		bool find_and_remove_elem( UINT16 requestid, CNQMessage **obj, UINT32 &lasterr );
		int	 size( UINT32 &lasterr );
		bool find_elem_ack_and_signal( const CNQMessage &aAckPacket, UINT32 &lasterr );
		bool find_and_remove_passed_elem( CNQMessage *obj, UINT32 &lasterr );


		// convenience functions
		bool setQueueMaxsize( INT32 maxsize);
		bool pop_front( CNQMessage **obj);
		bool push_back ( const CNQMessage* obj);
		bool remove_elem( UINT16 requestid);
		bool find_elem( UINT16 requestid, CNQMessage **obj);
		bool find_and_remove_elem( UINT16 requestid, CNQMessage **obj);
		int	 size();
		bool find_elem_ack_and_signal( const CNQMessage &aAckPacket);
		bool find_and_remove_passed_elem( CNQMessage *obj);

	private:
		INT32 m_iQueueFirstElement;		// -1 == no first element
		INT32 m_iQueueLastElement;		// -1 == no last element
		INT32 m_iQueueNumElements;
		
		VOIDP *m_pQueueValue;			// elements (0 == empty)
		INT32 *m_iQueueNextElement;		// ptr to next element

		INT32 m_iQueueMaxSize;			// queue maximum size

		UINT32 m_iLastError;			// last error that happened inside

};


#endif // __NQPACKETQUEUE_H_INCLUDED__
