#ifndef __PLATFORM_H_INCLUDED_SW__
#define __PLATFORM_H_INCLUDED_SW__

// + SW (Jul 2007 to Aug 2007)
//
//  merged all platform dependancy files .cpp and .h into one big Platform.h and Platform.cpp
//  to avoid redundancy and have better overview
//

#include "nq_system.h"

#ifdef SYSTEM_SUN
	#include <stdio.h>
	#include <stdlib.h>
	#include <errno.h>
	#include <string.h>
	#include <ctype.h>
	#include <termios.h>

	#include <stdarg.h>

	#include <sys/timeb.h>
	#include <sys/times.h>
	#include <sys/types.h>
	#include <unistd.h>
	#include <pthread.h>
	#include <sys/resource.h>
	#include <sys/file.h>					// FNONBLOCK FASYNC
	#define  PLF_MAX_FDSETS 		64	// taken from Phil, project.def, LINUX
	#define  PLF_ENDIANCHANGE
	#define  PLF_USING_PTHREAD
  #define  PLF_SYSTEM_SOLARIS
  
  #define  PLF_UNIX_TCPIPSTACK  
	#define  INADDR_NONE 			0xffffffff // for solaris only
#endif

#ifdef SYSTEM_LINUX
	#include <stdio.h>	
	#include <stdlib.h>
	#include <errno.h>
	#include <string.h>
	#include <ctype.h>
	#include <termios.h>

	#include <stdarg.h>

	#include <sys/timeb.h>
	#include <sys/times.h>
	#include <sys/types.h>
	#include <unistd.h>
	#include <pthread.h>
	#include <sys/resource.h>
	#include <sys/ioctl.h>  
	#define  PLF_MAX_FDSETS 64		// taken from Phil, project.def, LINUX
	#define  PLF_USING_PTHREAD

	#undef   PLF_ENDIANCHANGE
	#define  PLF_SYSTEM_LINUX

  #define  PLF_UNIX_TCPIPSTACK
#endif

#ifdef SYSTEM_WINDOWS
	#include <stdio.h>
	#include <windows.h>
	#include <conio.h>
	#include <time.h>

	#define SO_SENDBUF_SIZE 40960
	#define SO_RECVBUF_SIZE 81920
	
	#define  PLF_SYSTEM_WINNT
	#define  PLF_MAX_FDSETS FD_SETSIZE
	#undef   PLF_ENDIANCHANGE
  #define  PLF_WINNT_TCPIPSTACK
#endif

#ifdef SYSTEM_NIOS2_LWIP

  #include <unistd.h>     // function to close a socket
  #include <stdio.h>
  #include <stdlib.h>
  #include <errno.h>
  #include <string.h>
  #include <ctype.h> 

  #include <sys/timeb.h>
  #include <sys/times.h>
  #include <sys/types.h>
  #include <sys/resource.h>

  // platform-specific include (required for ioctl function)
  #include <sys/ioctl.h> 
  #include <time.h>
  #include <fcntl.h>
  
  #ifdef __cplusplus
  extern "C" {
  #endif

  #include "includes.h"
  #include "alt_lwip_dev.h"
  #include "lwip/sys.h"
  #include "lwip/sockets.h"
  #include "user.h"

  #ifdef __cplusplus
  }
  #endif

  #define  PLF_SYSTEM_NIOS2
  #define  PLF_MAX_FDSETS  FD_SETSIZE

  #define  PLF_LWIP_TCPIPSTACK
  #define  BINLOCK_SEM_TIMEOUT 100
	#undef   PLF_ENDIANCHANGE
  
  // NIOS2 stack size (in bytes) for send and receive tasks
  #define TASK_STACK_SIZE_SW 32768
  //#define TASK_STACK_CHECKING_SW OS_TASK_OPT_STK_CHK
  #define TASK_STACK_CHECKING_SW 0

  #define SENDTASK_PRIO_SW     8
  #define RECVTASK_PRIO_SW     7
  #define ETHER_PRIO_SW        5 
  #define TCPIP_PRIO_SW        6  
  // SW! recvtask_prio and sendtask_prio see platform.h
  #define MAIN_PRIO_SW         9

/*
  #define SENDTASK_PRIO_SW     11
  #define RECVTASK_PRIO_SW     8
  #define TCPIP_PRIO_SW        7
  #define ETHER_PRIO_SW        6 
  // SW! recvtask_prio and sendtask_prio see platform.h
  #define MAIN_PRIO_SW         10
  */  
#endif

#ifdef SYSTEM_NIOS2_NICHE

  #include <unistd.h>     // function to close a socket
  #include <stdio.h>
  #include <stdlib.h>
  #include <errno.h>
  #include <string.h>
  #include <ctype.h> 

  #include <sys/timeb.h>
  #include <sys/times.h>
  #include <sys/types.h>
  #include <sys/resource.h>

  // platform-specific include (required for ioctl function)
  #include <sys/ioctl.h> 
  #include <time.h>
  #include <fcntl.h>
  
  #ifdef __cplusplus
    extern "C" {
  #endif

    #include "includes.h"
    #include <unistd.h>
    #include "system.h"

    /* Nichestack definitions */
    #include "ipport.h"
    #include "libport.h"
    #include "osport.h"
    #include "tcpport.h"
    #include "bsdsock.h"

  #ifdef __cplusplus
    }
  #endif


  #define  PLF_SYSTEM_NIOS2
  #define  PLF_MAX_FDSETS  FD_SETSIZE

  #define  PLF_NICHE_TCPIPSTACK
  #define  BINLOCK_SEM_TIMEOUT 100
  #undef   PLF_ENDIANCHANGE
  
  // NIOS2 stack size (in bytes) for send and receive tasks
  #define TASK_STACK_SIZE_SW 32768
  //#define TASK_STACK_CHECKING_SW OS_TASK_OPT_STK_CHK
  #define TASK_STACK_CHECKING_SW 0

  #define INITIAL_TASK_PRIO 1 // niche SW! May 10, 2007
  // niche: prios 2 and 3 are required internally
  
  #define SENDTASK_PRIO_SW     8
  #define RECVTASK_PRIO_SW     7
//  #define ETHER_PRIO_SW        5 
//  #define TCPIP_PRIO_SW        6  
  // SW! recvtask_prio and sendtask_prio see platform.h
  #define MAIN_PRIO_SW         9

/*
  #define SENDTASK_PRIO_SW     11
  #define RECVTASK_PRIO_SW     8
  #define TCPIP_PRIO_SW        7
  #define ETHER_PRIO_SW        6 
  // SW! recvtask_prio and sendtask_prio see platform.h
  #define MAIN_PRIO_SW         10
  */  
  #ifndef INADDR_NONE
    #define  INADDR_NONE            0xffffffff // for solaris only
  #endif
  #ifndef AF_UNSPEC
    #define AF_UNSPEC 0
  #endif
    
#endif

#include "TypesSW.h"

#ifdef __NQ_INTERNAL_SW__

#ifndef PLF_SYSTEM_WINNT
	#ifdef _SW_RELEASE_
		#define NDEBUG
	#endif
#endif

 
#define __NEED_SOCKET_FUNCTIONS_SW__
#define __NEED_SEMAPHORE_SW__
#define __NEED_MUTEX_SW__
#define __CREATE_TASK_THREAD__


#ifdef __NEED_SOCKET_FUNCTIONS_SW__
	#ifdef PLF_UNIX_TCPIPSTACK
		#include <sys/socket.h> // core socket function
		#include <sys/types.h>
		#include <arpa/inet.h>  // some additional defines for internet (tcp/ip) sockets 
		#include <netinet/in.h> // internet structures for socket
		#include <netdb.h>      // gethostbyname
		#include <fcntl.h>
	#endif
#endif // #ifdef __NEED_SOCKET_FUNCTIONS_SW__


#if defined(PLF_LWIP_TCPIPSTACK) || defined(PLF_UNIX_TCPIPSTACK) 
	// win32, niche: already defined
	extern const	INT32	SOCKET_ERROR;
	extern const	INT32	INVALID_SOCKET;
#endif

#ifdef PLF_WINNT_TCPIPSTACK
	#define EADDRNOTAVAIL	-3
	#define EAFNOSUPPORT	-4
	#define EALREADY		-5
	#define ECONNREFUSED	-7
	#define EINPROGRESS		-8

	#define EISCONN			-10
	#define ENETUNREACH		-11
	#define ENOTSOCK		-12
	#define EPROTOTYPE		-13
	#define ETIMEDOUT		-14
	#define EADDRINUSE		-15
	#define ECONNRESET		-16
	#define EHOSTUNREACH	-17
	#define ENETDOWN		-20
	#define EUNKNOWN		-22 
	#define EHOSTDOWN		-23

	// are already declared on Windows somewhen, so wrap accordingly
	#ifndef EBADF
		#define EBADF			-6
		#define EAGAIN			-21 
		#define EINVAL			-18
		#define ENAMETOOLONG	-19
		#define EINTR			-9
	#endif

#endif

#ifdef __NEED_SOCKET_FUNCTIONS_SW__

// function declarations required 

extern 		void		platformInitializeSocketSubsystem();
extern 		void		platformDeinitializeSocketSubsystem();

extern		INT32		platformSocketGetLastError();
extern		socktype	platformSocketCreateInet( INT32 type );

//extern		INT32		platformSocketReceiveFunction( socktype s, UINT8 *buf, UINT32 buflen);
//extern		INT32		platformSocketSendFunction( socktype s, const UINT8 *buf, UINT32 buflen);

extern		INT32		platformSocketReceiveFromFunction( socktype s, UINT8 *buf, UINT32 buflen, UINT32 *ip_hostorder, UINT16 *port_hostorder);
extern		INT32		platformSocketSendToFunction( socktype s, const UINT8 *buf, UINT32 buflen, UINT32 ip_hostorder, UINT16 port_hostorder );

extern		INT32		platformSocketClose( socktype s);
extern		INT32		platformSocketConnect(socktype s, const struct sockaddr * Saddr, size_t size_sockaddr);
extern		INT32		platformSocketBind( socktype s, const struct sockaddr * Saddr, size_t size_sockaddr);
extern		INT32		platformSocketSetNonblocking ( socktype s );
extern		UINT32		platformIPAddrConvert( const ONECHAR *ip_addr_dotted_string);
extern 		ONECHAR		*platformIPAddrConvert( const UINT32 ip_hostorder );
extern		INT32		platformSocketDisconnectUDP( socktype s);
extern		INT32		platformSocketSelect(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);

extern const INT32 platformMaxFdSets;

#endif // #ifdef __NEED_SOCKET_FUNCTIONS_SW__

extern HANDLE	platformCreateSemaphore( UINT16 maximum_count );
extern bool		platformDeleteSemaphore( HANDLE semaph );
extern bool		platformWaitForSemaphore( HANDLE semaph, UINT32 timeout_msec );
extern bool		platformSignalSemaphore( HANDLE semaphore );

extern HANDLE	platformCreateMutex();
extern bool		platformDeleteMutex( HANDLE mutex );
extern bool		platformWaitForMutex( HANDLE mutex, UINT32 timeout_msec );
extern bool		platformSignalMutex( HANDLE mutex );
extern bool		platformIsSemaphoreSignalled( HANDLE semaph );

#define			SW_THREAD_PRIORITY_ABOVE_NORMAL		1
#define			SW_THREAD_PRIORITY_NORMAL			2
#define			SW_THREAD_PRIORITY_BELOW_NORMAL		3

extern HANDLE	platformCreateThread( bool sendthread, VOIDP parameter, UINT32 thread_prio_sw);
extern bool		platformDeleteThread( HANDLE threadhandle );

#ifndef PLF_SYSTEM_WINNT
	extern int	_strnicmp( const char *string1, const char *string2, size_t count);
#endif // NO PLF_SYSTEM_WINNT 

extern VOID		platformSleepMilliseconds(UINT32 milliseconds);
extern UINT32	platformGetTickCount();

extern UINT32	platformGetLastError();
extern ONECHAR  *platformGetOSErrorString( UINT32 intOSLastError, ONECHAR *lasterrorstr, UINT32 maxlength_lasterrorstr);

extern void		platformInitializeConsoleInput();
extern void		platformDeinitializeConsoleInput();
extern INT32	platformgetch();
extern INT32	platformkbhit();

#ifdef PLF_ENDIANCHANGE
  extern void	platform_put_double_at( UINT8 *bufpos, DOUBLE data );
  extern void	platform_put_float_at( UINT8 *bufpos, FLOAT data );
  extern void   platform_put_uint32_at( UINT8 *bufpos, UINT32 data );
  extern void   platform_put_uint16_at( UINT8 *bufpos, UINT16 data );
  extern void   platform_put_uint8_at( UINT8 *bufpos, UINT8 data );
  extern void   platform_put_int32_at( UINT8 *bufpos, INT32 data );
  extern void   platform_put_int16_at( UINT8 *bufpos, INT16 data );
  extern void   platform_put_int8_at( UINT8 *bufpos, INT8 data );
  extern FLOAT  platform_get_float_at( UINT8 *bufpos );
  extern DOUBLE platform_get_double_at( UINT8 *bufpos );
  extern UINT32 platform_get_uint32_at( UINT8 *bufpos );
  extern UINT16	platform_get_uint16_at( UINT8 *bufpos );
  extern UINT8  platform_get_uint8_at( UINT8 *bufpos );
  extern INT32  platform_get_int32_at( UINT8 *bufpos );
  extern INT16  platform_get_int16_at( UINT8 *bufpos );
  extern INT8   platform_get_int8_at( UINT8 *bufpos );
#else
  // endian conversion macros (just empty here at it is little by little)
  // extern void    platform_put_double_at( UINT8 *bufpos, DOUBLE data );
  #define platform_put_double_at(a,b)   *((DOUBLE *) (a)) = (DOUBLE) (b)
  // extern void		platform_put_float_at( UINT8 *bufpos, FLOAT data );
  #define platform_put_float_at(a,b)    *((FLOAT *) (a)) = (FLOAT) (b)
  // extern void    platform_put_uint32_at( UINT8 *bufpos, UINT32 data );
  #define platform_put_uint32_at(a,b)   *((UINT32 *) (a)) = (UINT32) (b)
  // extern void    platform_put_uint16_at( UINT8 *bufpos, UINT16 data );
  #define platform_put_uint16_at(a,b)   *((UINT16 *) (a)) = (UINT16) (b)
  // extern void    platform_put_uint8_at( UINT8 *bufpos, UINT8 data );
  #define platform_put_uint8_at(a,b)    *((UINT8 *) (a)) = (UINT8) (b)
  // extern void    platform_put_int32_at( UINT8 *bufpos, INT32 data );
  #define platform_put_int32_at(a,b)    *((INT32 *) (a)) =(INT32) (b)
  // extern void    platform_put_int16_at( UINT8 *bufpos, INT16 data );
  #define platform_put_int16_at(a,b)    *((INT16 *) (a)) = (INT16) (b)
  // extern void    platform_put_int8_at( UINT8 *bufpos, INT8 data );
  #define platform_put_int8_at(a,b)     *((INT8 *) (a)) = (INT8) (b)
  
  // extern FLOAT   platform_get_float_at( UINT8 *bufpos );
  #define platform_get_float_at(a)      *((FLOAT *) (a))
  // extern DOUBLE  platform_get_double_at( UINT8 *bufpos );
  #define platform_get_double_at(a)   *((DOUBLE *) (a))
  // extern UINT32  platform_get_uint32_at( UINT8 *bufpos );
  #define platform_get_uint32_at(a)   *((UINT32 *) (a))
  // extern UINT16	platform_get_uint16_at( UINT8 *bufpos );
  #define platform_get_uint16_at(a)   *((UINT16 *) (a))
  // extern UINT8   platform_get_uint8_at( UINT8 *bufpos );
  #define platform_get_uint8_at(a)    *((UINT8 *) (a))
  // extern INT32   platform_get_int32_at( UINT8 *bufpos );
  #define platform_get_int32_at(a)    *((INT32 *) (a))
  // extern INT16   platform_get_int16_at( UINT8 *bufpos );
  #define platform_get_int16_at(a)    *((INT16 *) (a))
  // extern INT8    platform_get_int8_at( UINT8 *bufpos );
  #define platform_get_int8_at(a)     *((INT8 *) (a))
#endif

#endif //__NQ_INTERNAL_SW__

#endif //__PLATFORM_H_INCLUDED_SW__
