#include "PlatformSW.h"

#ifndef ALT_INICHE
  #error This Web Server requires the Interniche IP Stack Software Component.
#endif

#ifndef __ucosii__
  #error This Web Server requires the UCOS II IP Software Component.
#endif

using namespace std;

extern void SimulateRemotePeer(const char *aIPAddressOtherPeer, UINT16 aPortOtherPeer, UINT16 aPort);
extern void SimulateLocalPeer(const char *aIPAddressOtherPeer, UINT16 aPortOtherPeer, UINT16 aPort);

void PrintoutTaskStatistics()
{
    OS_TCB tcb;
    int j;
    
    for(j=1;j<20;j++) 
    {         
        // An SW:    Eventuell hier die PRIOs ndern, welche Du abfragen willst
        if ( OSTaskQuery (j, &tcb) == OS_NO_ERR) 
        {
            printf("%d:%s\tSwitchCtx:%lu\tCyclesRunning:%lu\t Stacksize:%lu\tStackUsed:%lu\n",
                    j,
                    tcb.OSTCBTaskName,
                    tcb.OSTCBCtxSwCtr,    // Number of time the task was switched in                     
                    tcb.OSTCBCyclesTot,   // Total number of clock cycles the task has been running      
                    tcb.OSTCBStkSize*sizeof(OS_STK),
                    tcb.OSTCBStkUsed      // Number of bytes used from the stack                         
                );
        }
    }
}


void LocalPeerStart()
{
  printf("Simulating (nios2 109 (vs. pi 13)) local peer...\n");
  SimulateLocalPeer("141.34.30.13", 31900, 32900);
}

void RemotePeerStart()
{
  printf("Simulating (nios2 109 (va. ...)) remote peer...\n");
  printf("No remote peer implemented.\n");
}

/*
 * void LocalPeerStart()
{
  printf("Simulating (nios2 109 (vs. oreade20 177)) local peer...\n");
  SimulateLocalPeer("141.34.30.177", 39900, 32900);
}

void RemotePeerStart()
{
  printf("Simulating (nios2 109 (va. ...)) remote peer...\n");
  printf("No remote peer implemented.\n");
}

 * 
 */

void main_task(void*)
{
    printf("Nios2 Niche Main() starting up...\n");
    
    //RemotePeerStart();
    LocalPeerStart();
  
    OSTaskDel(OS_PRIO_SELF);
}

#define MAINTASK_STACK_SIZE 16384*4

TK_OBJECT(to_maintask);
TK_ENTRY(main_task);

struct inet_taskinfo maintask = {
      &to_maintask,
      "main task (network queue)",
      main_task,
      MAIN_PRIO_SW,
      MAINTASK_STACK_SIZE,
};

#define TASK_STACKSIZE 2048

OS_STK    InitialTaskStk[TASK_STACKSIZE];

#if OS_STK_GROWTH == 0
#error os_stk_growth == 0 (must be != 0)
#endif

void InitialTask(void* pdata)
{
  INT8U error_code = OS_NO_ERR;

  /*
  * Initialize Altera NicheStack TCP/IP Stack - Nios II Edition specific code.
  * NicheStack is initialized from a task, so that RTOS will have started, and 
  * I/O drivers are available.  Two tasks are created:
  *    "Inet main"  task with priority 2
  *    "clock tick" task with priority 3
  */
  
  OSStatInit();
     
  alt_iniche_init();
  /* Start the Iniche-specific network tasks and initialize the network
   * devices.
   */
  netmain(); 
  /* Wait for the network stack to be ready before proceeding. */
  while (!iniche_net_ready)
    TK_SLEEP(1);
    
  /* Create the main network task.  In this case, a web server. */
  TK_NEWTASK(&maintask);

  /* Application specific code starts here... */
  
  /*Create Tasks*/
  //CreateTasks();
  //printf("\nwebserver is starting up\n");
  /* Application specific code ends here. */

  /*This task deletes itself, since there's no reason to keep it around, once
   *it's complete.
   */
  error_code = OSTaskDel(OS_PRIO_SELF);

  while(1); /*Correct Program Flow should not reach here.*/
}

int main ()
{
  printf("\n\nNetworkQueue(NIOS2/ucOS-II/niche) project starting up\n\n");
  //printf("FD_SETSIZE=%d\n",FD_SETSIZE);
  /* 
   * Our very first call in an LWIP example design is to start up the LWIP 
   * stack. We specify a MicroC/OS-II thread priority for the tcp_ip thread,
   * as well as a call-back routine, tcpip_init_done(), which is called once
   * the stack is alive and well.
   */
  
  INT8U error_code;

  /* Clear the RTOS timer */
  OSTimeSet(0);
  
  /* WSInitialTask will initialize the NicheStack TCP/IP Stack and then 
   * initialize the rest of the web server's tasks.
   */ 

  error_code = OSTaskCreateExt(InitialTask,
                             NULL,
                             (OS_STK *) &InitialTaskStk[TASK_STACKSIZE-1],
                             INITIAL_TASK_PRIO,
                             INITIAL_TASK_PRIO,
                             InitialTaskStk,
                             TASK_STACKSIZE,
                             NULL,
                             0);
  
  /*
   * As with all MicroC/OS-II designs, once the initial thread(s) and 
   * associated RTOS resources are declared, we start the RTOS. That's it!
   */
  
  // activate statistic task
    
  OSStart();
  
  while(1); /* Correct Program Flow never gets here. */

  return -1;
}
