#ifndef __DECLARATIONS_H_INCLUDED__
#define __DECLARATIONS_H_INCLUDED__

#include "PlatformSW.h"



//! Maximal UDP Packet Size that is possible to send.
#define NQ_MAX_PACKET_SIZE				1472

//! Default size of packet queue
#define NQ_DEFAULT_PACKETQUEUE_SIZE	10

// error definitions inside classes 


/** \defgroup nq_errorcodes Network Queue Error Codes
 *  \brief 
 *  When handling with error codes, please \b always use the constant definitions name instead of real number!   
 *  @{
 */

// CNQPeerList
/** @name Error Codes of CNQPeerList
 *  @{
 **/

//! No error
#define ERR_PL_NONE				0			
//! Mutal lock of a critical section failed
#define ERR_PL_LOCK				1
//! Data storage buffer is full
#define ERR_PL_FULL				2			
//! Element not found in buffer
#define ERR_PL_NOTFOUND			3
//! Receive packet index is not (yet) set
#define ERR_PL_INDEXNOTSET		4

/** @}*/ 

    
// CNQPacketQueue
/** @name Error Codes for CNQPacketQueue
 *  @{
 **/

//! No error
#define ERR_PQ_NONE				0	
//! Mutal lock of a critical section failed
#define ERR_PQ_LOCK				1	
//! Data storage buffer is full
#define ERR_PQ_FULL				2	
//! Element not found in buffer
#define ERR_PQ_NOTFOUND			3	
//! Bad parameter passed
#define ERR_PQ_BADPARAM			4	

/* @}*/

// CNQResponseWaitQueue
//! @name Error Codes for CNQResponseWaitQueue
/* @{ */

//! no error
#define ERR_RWQ_NONE			0
//! Mutal Lock of a critical section failed
#define ERR_RWQ_LOCK			1
//! Data storage buffer is full
#define ERR_RWQ_FULL			2
//! Element not found in buffer
#define ERR_RWQ_NOTFOUND		3

/* @} */

// CNetworkQueue
#define ERR_NQ_BASE      0
#define ERR_NQ_NUMERRORS 28

/*  @name Error Codes for CNetworkQueue
 *  @{
 */
//! No error
#define ERR_NQ_NONE				(ERR_NQ_BASE+0)
//! Queue is already connected
#define ERR_NQ_ALREADY  		(ERR_NQ_BASE+1)
//! CNQMessage pool buffers could not be properly initialized
#define ERR_NQ_BADPOOL  		(ERR_NQ_BASE+2)
//! Malformed string IP passed
#define ERR_NQ_BADIP			(ERR_NQ_BASE+3)
//! UDP socket could not be created properly
#define ERR_NQ_SOCREATE 		(ERR_NQ_BASE+4)
//! UDP socket bind unsuccessful
#define ERR_NQ_SOBIND			(ERR_NQ_BASE+5)
//! initial peer could not be added at init time
#define ERR_NQ_ADDPEER  		(ERR_NQ_BASE+6)
//! error setting socket to nonblocking
#define ERR_NQ_SONONBLOCK 		(ERR_NQ_BASE+7)
//! send semaphore create error
#define ERR_NQ_SEMCREATESEND	(ERR_NQ_BASE+8)
//! receive semaphore create error
#define ERR_NQ_SEMCREATERECV	(ERR_NQ_BASE+9)
//! could not start send thread
#define ERR_NQ_THRDCREATESEND	(ERR_NQ_BASE+10)
//! could not start receive thread
#define ERR_NQ_THRDCREATERECV	(ERR_NQ_BASE+11)
//! thread cleanup failed
#define ERR_NQ_THRDCLEANUP		(ERR_NQ_BASE+12)
//! socket cleanup failed
#define ERR_NQ_SOCLOSE			(ERR_NQ_BASE+13)
//! semaphore cleanup
#define ERR_NQ_SEMCLEANUP		(ERR_NQ_BASE+14)
//! invalid message
#define ERR_NQ_MSGINVALID		(ERR_NQ_BASE+15)
//! internal send queue stuffed
#define ERR_NQ_SENDQUEUEFULL	(ERR_NQ_BASE+16)
//! internal send queue error
#define ERR_NQ_SENDQUEUE		(ERR_NQ_BASE+17)
//! semaphore notification failed
#define ERR_NQ_SEMNOTIFY		(ERR_NQ_BASE+18)
//! empty queue, no element returned
#define ERR_NQ_EMPTY			(ERR_NQ_BASE+19)
//! internal Receive Queue error
#define ERR_NQ_RECVQUEUE		(ERR_NQ_BASE+20)
//! timeout waiting for response / ack
#define ERR_NQ_TIMEOUT			(ERR_NQ_BASE+21)
//! temporary semaphore could not be created
#define ERR_NQ_SEMCREATE		(ERR_NQ_BASE+22)
//! response could not be found in special queue
#define ERR_NQ_RESPTROUBLE		(ERR_NQ_BASE+23)
//! temporary semaphore could not be deleted
#define ERR_NQ_SEMDELETE		(ERR_NQ_BASE+24)
//! invalid, unknown peer passed
#define ERR_NQ_INVALIDPEER		(ERR_NQ_BASE+25)
//! internal error in ack packet waiting queue
#define ERR_NQ_ACKQUEUE			(ERR_NQ_BASE+26)
//! internal error in responsewait queue
#define ERR_NQ_RESPWAITQUEUE	(ERR_NQ_BASE+27)

/**@}*/

/**@}*/ // Group nq_errorcodes


/** \defgroup nq_internal_defs Internal Definitions
 *  @{
 */

// -------- from message.h

/** @name Physical Type
 *  \sa \ref packet_types_table
 * @{
 */
#define NQ_PT_NONE					0x00
#define NQ_PT_REQUEST				0x01
#define NQ_PT_RESPONSE				0x02
#define NQ_PT_TELEGRAM				0x03

/** @} */

/** @name Logical Type
 *  \sa packet_types_table
 *  @{
 **/
//! \sa packet_format
//! No Logical Type defined
#define NQ_LT_NONE					0x00
//! Logical 'get' paket (request data from peer) 
#define NQ_LT_GET					0x01
//! Logical 'put' paket (write data to peer)
#define NQ_LT_PUT					0x02
//! Logical 'call' paket (call a remote function on peer)
#define NQ_LT_CALL					0x03
//! Logical 'trap' paket (signal something to peer)
#define NQ_LT_TRAP					0x04
//! Logical 'update' paket (a self inducted paket, that was send to update data at remote peer)
#define NQ_LT_UPDATE				0x05
/** @}*/

// only last two bits are defined
#define NQ_PACKET_SPECIAL_BITS_VALID_MASK		0xFFFC
#define NQ_PACKET_SPECIAL_BITS_DEFAULT_VALUE	0x0000
#define NQ_PACKET_BIT_WANT_ACK					0x0002
#define NQ_PACKET_BIT_IS_ACKPACKET				0x0001

#define NQ_MIN_VARTYPE					1
#define NQ_MAX_VARTYPE					(NQ_VARTYPE_TEXT)

/** @name Variable Types
 * 
 *  @note NQ_VARTYPE_BINARY and NQ_VARTYPE_TEXT are naturally 1-dimensional arrays by itself\n
 *  other VARTYPES can be also 1-dimensional arrays (in shortest case 1 element)
 *
 *  \see CNQMessage::write CNQMessage::read 
 *  @{
 **/
//! Variable type constant to identify \b binary data 
#define NQ_VARTYPE_BINARY				(NQ_MIN_VARTYPE)
//! Variable type constant to identify \b float data  
#define NQ_VARTYPE_FLOAT				(NQ_MIN_VARTYPE+1)
//! Variable type constant to identify \b double data  
#define NQ_VARTYPE_DOUBLE				(NQ_MIN_VARTYPE+2)
//! Variable type constant to identify <b> signed 8bit </b> data  
#define NQ_VARTYPE_INT8					(NQ_MIN_VARTYPE+3)
//! Variable type constant to identify <b> unsigned 8bit </b> data  
#define NQ_VARTYPE_UINT8				(NQ_MIN_VARTYPE+4)
//! Variable type constant to identify <b> signed 16bit </b> data  
#define NQ_VARTYPE_INT16				(NQ_MIN_VARTYPE+5)
//! Variable type constant to identify <b> unsigned 16bit </b> data  
#define NQ_VARTYPE_UINT16				(NQ_MIN_VARTYPE+6)
//! Variable type constant to identify <b> signed 32bit </b> data  
#define NQ_VARTYPE_INT32				(NQ_MIN_VARTYPE+7)
//! Variable type constant to identify <b> unsigned 32bit </b> data  
#define NQ_VARTYPE_UINT32				(NQ_MIN_VARTYPE+8)
//! Variable type constant to identify \b textual data  
#define NQ_VARTYPE_TEXT					(NQ_MIN_VARTYPE+9)
/**@}*/

#define NQ_MAX_PACKETPAYLOAD_SIZE		(NQ_MAX_PACKET_SIZE-NQ_SIZE_MESSAGE_FILL_STUFF-NQ_SIZE_PACKET_FILL_STUFF)

//! min_size_data (inside linked list): 
#define NQ_MIN_SIZE_DATA				(16)

//! one data block on the net (minimum size)
#define NQ_MIN_SIZE_DATA_NET			(12)

//! maximum amount of data entries per packet
//! depends on max_packetload_size and min_size_data
#define NQ_MAX_NUM_ENTRIES				(NQ_MAX_PACKETPAYLOAD_SIZE / NQ_MIN_SIZE_DATA_NET)

//! size to add for in-CNQMessage workbuffer wor max_packetpayload_size (includes linked list costs)
#define NQ_SIZE_ADD_LINKEDLIST			(4*NQ_MAX_NUM_ENTRIES)

// -------- from Networkqueue.h
#define NQ_SENDQUEUE_SIZE				10
#define NQ_RECVQUEUE_SIZE				10
#define NQ_ACKQUEUE_SIZE				10
#define NQ_NUM_CNQMESSAGEBUFFERS			40

#define NQ_SEND_SEMAPHORE_TIMEOUT_MS	500
#define NQ_SEND_SELECT_TIMEOUT_MS		200
#define NQ_RECV_SELECT_TIMEOUT_MS		100

#define NQ_SEND_THREAD_PRIORITY		SW_THREAD_PRIORITY_BELOW_NORMAL
//#define RECV_THREAD_PRIORITY		SW_THREAD_PRIORITY_ABOVE_NORMAL // does not work on linux without root priv
#define NQ_RECV_THREAD_PRIORITY		SW_THREAD_PRIORITY_NORMAL

// (authid, pidx, reqid, subsysid, numdlbk, packet types, packet type param)
#define NQ_SIZE_MESSAGE_FILL_STUFF		16

// (magic, size, version, alignfiller, tail)
#define NQ_SIZE_PACKET_FILL_STUFF		10

// packet magics and supported version
#define NQ_PACKET_MAGIC_TOP			0xA51C
#define NQ_PACKET_MAGIC_TAIL			0xC15A
#define NQ_PACKET_VERSIONS_SUPPORTED	0x0001

#define NQ_NO_SEMAPHORE				NULL

// -------- for ResponseWaitQueue
#define NQ_RESPONSEWAITQUEUE_MAX_SIZE	10

// -------- for PeerList
#define NQ_PEERLIST_MAX_SIZE			10

/** @} */

// special timeout type used e.g. by CNetworkQueue
// platform-independent timeval
typedef struct
{
	INT32 tv_sec;	//seconds
	INT32 tv_usec;	//microseconds
} NQ_TIMEVAL; // PlatformIndependent TIMEVAL structure


//! Structure containing statistical data
typedef struct 
{    
    /** @name Receive statistics
     *  @brief happens in receive task
     *  @{
     */
    
	UINT32 RcvInvalidPeerAddress;

    //! zero or socket error returned at recv()
	UINT32 RcvSocketErrors;					
	UINT32 RcvNoBuffersLeft;	
    //! zero or socket error returned at recv()
	UINT32 RcvNoDataReceived;				
    //! socket error returned at select in ReceiveProcessTask()
	UINT32 RcvSelectErrors;					
	
    //! dropped packets on receive (failed integrity check)
	UINT32 RcvDroppedPacketsBadChecksum;	
    //! amount of telegrams definitely successfully received
	UINT32 RcvNumTelegrams;					
    //! amount of requests definitely successfully received 
	UINT32 RcvNumRequests;					
    //! amount of responses definitely successfully received  
	UINT32 RcvNumResponses;					  
	UINT32 RcvNumPacketsWantAck;  
	UINT32 RcvAckPackets;  
    //! as this is a peer to peer queue (1:1), each sent packet will get a
    //! index value (incremented by 1 for each packet) so on receive side
    //! it can be checked whether packets were lost on the network
	UINT32 RcvPacketsMissedByIndex;			

    //! responses that are answers to request sent by blocking call
    //! different semaphore to signal
	UINT32 RcvResponsesWithSemaphore;		
											
	//! asynchronous responses (usual receive queue)
	UINT32 RcvResponsesWithoutSemaphore;	

	//! queue is full, so packet is dropped silently
	UINT32 RcvDroppedPacketsQueueFull;		
    //! any queue function call returned error (false)
	UINT32 RcvQueueErrors;					
    
    /**@}*/

	/** @name Send statistics
	 *  @brief happens in send task
     *  @{
     */
	
	UINT32 SntTelegrams;					//! amount of telegrams definitely successfully sent 
	UINT32 SntRequests;						//! amount of requests definitely successfully sent 
	UINT32 SntResponses;					//! amount of responses definitely successfully sent 
	UINT32 SntAckPackets;                   //! amount of responses definitely successfully sent

	UINT32 SntSocketErrors;					//! socket error on send() function
	UINT32 SntEncodingErrors;				//! encoding error (logical -> physical packet)
	
	UINT32 SntQueueErrors;					//! queue erros in SendTask()
	UINT32 SntQueueFull;
    /*@}*/
	
	UINT32 BlockingCallResponseTimeout;
	UINT32 BlockingCallResponseReceivedOnTime;
	UINT32 BlockingCallRequestSent;

	UINT32 AckTimeout;
	
    //! start time of CNetworkQueue
	NQ_TIMEVAL StartTime;						

} NQ_STATISTICS;

//! required for synchronous call support
typedef struct
{
	UINT16	requestid;
	HANDLE	semaphore_to_signal;
} NQ_WAITFORRESPONSE;

/**@}*/ // group nq_internal_defs

/*

  !pseudocode packet structure!

typedef struct
{
	// general packet header
	UINT16	packet_magic;
	UINT16	packet_size;
	UINT16	packet_version;
	UINT16  packet_special_bits;

	///////////////////////// copy of message struct START

	// specific packet header, ver2
	UINT32	authorisation_id;
	UINT16	packet_index;
	UINT16	request_id;
	UINT16	subsystem_device_id;
	UINT16	num_datablocks;

	UCHAR	physical_packet_type;
	UCHAR	logical_packet_type;

	UINT16	packet_type_param;	// for TRAP type: contains TRAP_ID
								// for RESP type: contains error code

	// for packet header 2
	UCHAR	packet_data[MAX_MSG_SIZE-SIZE_MESSAGE_FILL_STUFF-SIZE_PACKET_FILL_STUFF];
	UINT16	packet_data_length; // in bytes


	///////////////////////// copy of message struct END

	UINT16	packet_tail;

	/////////////////////////////////////////////////////////
	HANDLE  notifySemaphoreOnReceive; 
} PACKET;
*/



#endif //__DECLARATIONS_H_INCLUDED__
