// MessageBuffers.h: interface for the CNQMessageBuffers class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __NQMESSAGEPOOL_H_INCLUDED__SW__
#define __NQMESSAGEPOOL_H_INCLUDED__SW__

/**  
	\brief Implementation of platform (and stack) independent message buffers of type CNQMessage
    
	Using C++ STL this could be done much more easily, but implementation on Nios2 platform did not work
	(reason not fully known, stack issues suspected). This CNQMessagePool was created as an inferior, but 
	working solution.

	Implementation details:
		- reference counting
		- clear() CNQMessage on putting back if reference count is zero afterwards
		- double array list

    \date  SW (Nov 2006 to Mar 2007)

  */
class CNQMessagePool : public CNQLockMechanism 
{
	friend class CNetworkQueue; // (-!-)

public:
    //! Constructor
	CNQMessagePool();
    //! Destructor
	virtual ~CNQMessagePool();

	//  initialize 'numbuffers' empty CNQMessage objects
	bool Initialize( UINT32 numbuffers );
	
	//  free memory, clean up anything
	bool Deinitialize();


protected:
	// (-!-)
	//  obtain a free (refcount 0) CNQMessage object
	CNQMessage *acquire_reference();
	
	//  put reference back (decrement reference count and clear CNQMessage if refcount is zero)
	bool putback_reference( CNQMessage *ref );
	
	//  increment reference count by 1 for a already taken CNQMessage (by get_reference())
	bool increment_referencecount( CNQMessage *buffer_ref );
	// (-!-)

private:
	bool	m_bInitialized;
	
	//! will assert on UnInitialize()
	UINT32	m_iSharedReferences;

	//! amount of CNQMessage buffers
	UINT32	m_iNumBuffers;
	
	//! 'numbuffers' messages
	CNQMessage *m_pCMessages;
	
	//! reference counter buffer (0 - not used)
	UINT32  *m_pTakenBuffer;

};

#endif // __NQMESSAGEPOOL_H_INCLUDED__SW__
