#ifndef __NQPEERLIST_H_INCLUDED__
#define __NQPEERLIST_H_INCLUDED__

#include "PlatformSW.h"
#include "NQLockMechanism.h"
#include "NQDeclarations.h"

/*
	CNQPeerList - SW (Jul 2007 to Aug 2007)

	Implementation of platform (and stack) independent 'peer list' queue
	
	Using C++ STL this could be done much more easily, but implementation on Nios2 platform did not work
	(reason not fully known, stack issues suspected). This class was created as a slightly inferior, but 
	working solution.

	Implementation details:
		- based on CNQResponseWaitQueue
		- double array linked list
		- access protected by binary semaphore or mutex object
		- enhanced error handling
*/


//
// NQ_PEER_ENTRY_INT - structure for holding internal per-peer information, like
// - ip and host of peer
// - last received packet index from peer (and a bool to define whether the field is already valid)
// - next to be used send packet index if sending a message to this peer
//
typedef struct
{
	bool	enabled;
	UINT32	ip_host; // host-order
	UINT16	port_host; // host-order
	bool	receive_last_packet_index_set;
	UINT16	receive_last_packet_index;
	UINT16	send_packet_index;
} NQ_PEER_ENTRY_INT;


class CNQPeerList: public CNQLockMechanism
{ // FIFO style
	public:
		CNQPeerList();
		virtual ~CNQPeerList();

		UINT32 getLastError();

		// convenience
		bool add( UINT32 ip, UINT16 port );
		bool remove( UINT32 ip, UINT16 port );
		bool removeAll();
		//bool getEntryByPos( UINT32 pos, UINT32 &ip, UINT32 &port);
		bool isAllowed(UINT32 ip, UINT16 port);
		//bool find( UINT32 ip, UINT16 port);
		bool getReceivePacketIndex( UINT32 ip, UINT16 port, UINT16 &packetindex);
		bool setReceivePacketIndex( UINT32 ip, UINT16 port, UINT16 packetindex);

		bool getSendPacketIndex( UINT32 ip, UINT16 port, UINT16 &aNextPacketIndex);
		bool incSendPacketIndex( UINT32 ip, UINT16 port);
		

		// low level

		UINT32 count();

		bool add( UINT32 ip, UINT16 port, UINT32 &lasterr);
		bool remove( UINT32 ip, UINT16 port, UINT32 &lasterr );
		bool removeAll( UINT32 &lasterr );
		
		//bool getEntryByPos( UINT32 pos, UINT32 &ip, UINT16 &port, UINT32 &lasterr);
		bool isAllowed(UINT32 ip, UINT16 port, UINT32 &lasterr);
		//bool find( UINT32 ip, UINT16 port, UINT32 &lasterr);
		
		bool getReceivePacketIndex( UINT32 ip, UINT16 port, UINT16 &packetindex, UINT32 &lasterr);
		bool setReceivePacketIndex( UINT32 ip, UINT16 port, UINT16 packetindex, UINT32 &lasterr);
		
		bool getSendPacketIndex( UINT32 ip, UINT16 port, UINT16 &aNextPacketIndex, UINT32 &lasterr );
		bool incSendPacketIndex( UINT32 ip, UINT16 port, UINT32 &lasterr);

	private:
		INT32 m_iQueueFirstElement;				// -1 == no first element
		INT32 m_iQueueLastElement;				// -1 == no last element
		INT32 m_iQueueNumElements;				// elements contained inside
		
		
		NQ_PEER_ENTRY_INT *m_pQueueValue;			// elements (0 == empty)
		INT32 *m_iQueueNextElement;				// ptr to next element

		INT32 m_iQueueMaxSize;					// maximum number of elements

		UINT32 m_iLastError;					// last error that happened inside
};



#endif // __NQPEERLIST_H_INCLUDED__
