#ifndef __NQRESPONSEWAITQUEUE_H_INCLUDED__
#define __NQRESPONSEWAITQUEUE_H_INCLUDED__

#include "PlatformSW.h"
#include "NQLockMechanism.h"
#include "NQDeclarations.h"

typedef struct
{
	UINT16 enabled;
	NQ_WAITFORRESPONSE wfr;
} NQ_WAITFORRESPONSEINT;


/*
	CNQResponseWaitQueue - SW (Nov 2006 to Mar 2007)

	Implementation of platform (and stack) independent FIFO packet queue for structs of type NQ_WAITFORRESPONSE.
	This special queue is required for synchronous calls (CNetworkQueue::SendRequestAndWaitForResponse()).
	
	Using C++ STL this could be done much more easily, but implementation on Nios2 platform did not work
	(reason not fully known, stack issues suspected). This CNQResponseWaitQueue was created as a slightly inferior, 
	but working solution.

	Implementation details:
		- double array linked list
		- access protected by binary semaphore or mutex object

*/


class CNQResponseWaitQueue : public CNQLockMechanism
{ // FIFO style
	public:
		CNQResponseWaitQueue();
		virtual ~CNQResponseWaitQueue();

		UINT32 getLastError();

		// convenience
		bool push_back ( const NQ_WAITFORRESPONSE& obj);
		bool remove_elem( UINT16 requestid);
		bool find_elem( UINT16 requestid, NQ_WAITFORRESPONSE& obj);
		bool find_and_remove_elem( UINT16 requestid, NQ_WAITFORRESPONSE& obj);
		
		// low level
		bool push_back ( const NQ_WAITFORRESPONSE& obj, UINT32 &lasterr );
		bool remove_elem( UINT16 requestid, UINT32 &lasterr );
		bool find_elem( UINT16 requestid, NQ_WAITFORRESPONSE& obj, UINT32 &lasterr );
		bool find_and_remove_elem( UINT16 requestid, NQ_WAITFORRESPONSE& obj, UINT32 &lasterr );
		
		UINT32 size();
		
	private:
		INT32 m_iQueueFirstElement;				// -1 == no first element
		INT32 m_iQueueLastElement;				// -1 == no last element
		INT32 m_iQueueNumElements;				// elements contained inside
		
		NQ_WAITFORRESPONSEINT *m_pQueueValue;		// elements (0 == empty)
		INT32 *m_iQueueNextElement;				// ptr to next element

		INT32 m_iQueueMaxSize;					// maximum number of elements

		UINT32 m_iLastError;					// last error that happened inside
};



#endif // __NQRESPONSEWAITQUEUE_H_INCLUDED__
